# -*- coding: utf-8 -*-
"""
Demonstrates GLVolumeItem for displaying volumetric data.

"""

import numpy as np
import importlib.util
import sys
import matplotlib.pyplot as plt
import importlib
import scipy as sp
import scipy.ndimage
import numpy as np
import time
from math import *
from pyqtgraph.Qt import QtCore, QtGui
import pyqtgraph.opengl as gl

def loadstack(filename):
    spec = importlib.util.spec_from_file_location("stackmod", filename)
    foo = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(foo)
    return foo

def newvalue(d2, lower, upper):
    t = (int(0.3 * d2[0]) + int(0.59 * d2[1]) + int(0.11 * d2[2])) / 3
    if t < lower:
        return [255, 0, 0, 0]
    elif t < upper:
        return [0, 255, 0, 100]
    else:
        return [0, 0, 255, 150]
    #return [t, t, t, d2[3]]

def makeGlVol(data, lower, upper):
    maxval = data.max()
    minval = data.min()
    # data = np.fromfunction(psi, (100, 100, 200))
    positive = np.clip(data, 0.000001, maxval) ** 2
    #positive = np.log(pclip)
    negative = np.clip(-data, 0.000001, -minval) ** 2
    #negative = np.log(nclip)

    d2 = np.empty(data.shape + (4,), dtype=np.ubyte)
    d2[..., 0] = positive * (255. / positive.max())
    d2[..., 1] = negative * (255. / negative.max())
    d2[..., 2] = d2[..., 1]
    d2[..., 3] = d2[..., 0] * 0.3 + d2[..., 1] * 0.3
    d2[..., 3] = (d2[..., 3].astype(float) / 255.) ** 2 * 255
    for x in range(0, len(d2)):
        for y in range(0, len(d2[0])):
            for z in range(0, len(d2[0][0])):
                d2[x][y][z] = newvalue(d2[x][y][z], lower, upper)

    #d2[:, 0, 0] = [255, 0, 0, 10]
    #d2[0, :, 0] = [0, 255, 0, 10]
    #d2[0, 0, :] = [0, 0, 255, 10]

    v = gl.GLVolumeItem(d2, smooth=False)

    return v

def render(stack_file, lower, upper):
    w = gl.GLViewWidget()
    w.opts['distance'] = 200
    w.show()
    w.setWindowTitle('pyqtgraph example: GLVolumeItem')

    # loads the list object containing the 3D stack matrix from a given file name
    #frame = 1

    stackmod = loadstack(stack_file)

    stack = stackmod.stack
    datazxy = np.array(stack)
    #dimmargin = (0,0,0) # used to trim the data by n elements around an axis by its index, ie. (1,0,0) cuts the first and last element on the x-axis for every matrix
    data = np.swapaxes(datazxy, 0, 2)
    #pscale = 10000000

    #scale = (stackmod.xi['step']*pscale, stackmod.yi['step']*pscale, stackmod.zi['step']*pscale)
    dims = data.shape
    g = gl.GLGridItem()
    g.scale(1, 1, 1)
    g.setSize(len(data), len(data[0]), len(data[0][0]))
    w.addItem(g)

    # example data conversion function from the original example
    v = makeGlVol(data, lower, upper)

    # translate the item to the middle
    v.translate(-dims[0]/2, -dims[1]/2, 0)
    #v.translate(-dims[0]*scale[0]/2, -dims[1]*scale[1]/2, 0)
    #and scale to the same aspect ratio from the metadata provided by the stack data
    #v.scale(scale[0], scale[1], scale[2])
    w.addItem(v)
    ax = gl.GLAxisItem()
    w.addItem(ax)
    #print(w.size())
    #x = np.linspace(0, dims[0], dims[0])
    #y = np.linspace(0, dims[1], dims[1])
    #z = np.ndarray(shape=(dims[0], dims[1]))



## Start Qt event loop unless running in interactive mode.
def main(stack_file, lower, upper):
    import sys
    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
        render(stack_file, lower, upper)


